

import inspect
import unittest

import glasswall
from glasswall.content_management.config_elements.config_element import ConfigElement
from glasswall.content_management.errors.config_elements import ConfigElementNotFound
from glasswall.content_management.errors.switches import SwitchNotFound
from glasswall.content_management.policies.policy import Policy
from glasswall.content_management.switches.switch import Switch


class TestPolicy(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        self.custom_policy = Policy(
            config_elements=[
                ConfigElement(
                    name="a",
                    switches=[
                        Switch(
                            name="b",
                            value="c",
                        ),
                        Switch(
                            name="d",
                            value="e",
                            attributes={"f": "g"},
                        )
                    ],
                ),
                ConfigElement(
                    name="pdfConfig",
                    switches=[
                        Switch(
                            name="embedded_files",
                            value="disallow",
                        ),
                        Switch(
                            name="embedded_images",
                            value="allow",
                        ),
                    ],
                ),
            ]
        )

        self.custom_policy_duplicates = Policy(
            config_elements=[
                ConfigElement(
                    name="a",
                    switches=[
                        Switch(
                            name="b",
                            value="c",
                        ),
                        Switch(
                            name="d",
                            value="e",
                            attributes={"f": "g"},
                        )
                    ],
                ),
                ConfigElement(
                    name="a",
                    switches=[
                        Switch(
                            name="b",
                            value="c",
                        ),
                        Switch(
                            name="embedded_images",
                            value="allow",
                        ),
                    ],
                ),
                ConfigElement(
                    name="pdfConfig",
                    switches=[
                        Switch(
                            name="embedded_files",
                            value="disallow",
                        ),
                        Switch(
                            name="embedded_images",
                            value="allow",
                        ),
                    ],
                ),
            ]
        )

    def tearDown(self):
        pass

    def test_text___custom_policy___text_matches_expected(self):
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """)
        )

    def test_text___custom_policy_duplicates___text_matches_expected(self):
        self.assertEqual(
            self.custom_policy_duplicates.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <a>
                    <b>c</b>
                    <embedded_images>allow</embedded_images>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """)
        )

    def test_encoded___return_matches_expected(self):
        self.assertEqual(
            self.custom_policy.encode(),
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """).encode("utf-8")
        )

    def test_get_config_element_names___no_duplicates___list_matches_expected(self):
        self.assertListEqual(
            self.custom_policy.get_config_element_names(),
            ["a", "pdfConfig"]
        )

    def test_get_config_element_names___with_duplicates___list_matches_expected(self):
        # Duplicate names not repeated
        self.assertListEqual(
            self.custom_policy_duplicates.get_config_element_names(),
            ["a", "pdfConfig"]
        )

    def test_remove_switch___config_element_not_found___raise_ConfigElementNotFound(self):
        with self.assertRaises(ConfigElementNotFound):
            self.custom_policy.remove_switch(config_element="splat", switch="asd")

    def test_remove_switch___switch_not_found___raise_SwitchNotFound(self):
        with self.assertRaises(SwitchNotFound):
            self.custom_policy.remove_switch(config_element="pdfConfig", switch="splat")

    def test_remove_switch___config_element_by_str_switch_by_str___switch_removed(self):
        self.custom_policy.remove_switch(config_element="pdfConfig", switch="embedded_images")

        # embedded_images no longer in config element "pdfConfig" switches
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                </pdfConfig>
            </config>
            """)
        )

    def test_remove_switch___config_element_by_str_switch_by_instance___switch_removed(self):
        switch_embedded_images = self.custom_policy.config_elements[1].switches[1]
        self.custom_policy.remove_switch(config_element="pdfConfig", switch=switch_embedded_images)

        # embedded_images no longer in config element "pdfConfig" switches
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                </pdfConfig>
            </config>
            """)
        )

    def test_remove_switch___config_element_by_instance_switch_by_str___switch_removed(self):
        config_element_pdfConfig = self.custom_policy.config_elements[1]
        self.custom_policy.remove_switch(config_element=config_element_pdfConfig, switch="embedded_images")

        # embedded_images no longer in config element "pdfConfig" switches
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                </pdfConfig>
            </config>
            """)
        )

    def test_remove_switch___config_element_by_instance_switch_by_instance___switch_removed(self):
        config_element_pdfConfig = self.custom_policy.config_elements[1]
        switch_embedded_images = self.custom_policy.config_elements[1].switches[1]
        self.custom_policy.remove_switch(config_element=config_element_pdfConfig, switch=switch_embedded_images)

        # embedded_images no longer in config element "pdfConfig" switches
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                </pdfConfig>
            </config>
            """)
        )

    def test_remove_switch___config_element_by_str_switch_by_str_with_duplicates___duplicates_switches_removed(self):
        # remove switch "embedded_images" from all config "a"
        self.custom_policy_duplicates.remove_switch(config_element="a", switch="embedded_images")

        # all config "a" no longer contain switch "embedded_images"
        self.assertEqual(
            self.custom_policy_duplicates.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <a>
                    <b>c</b>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """)
        )

    def test_add_switch___config_element_by_str_not_found___raise_SwitchNotFound(self):
        # Try to add a switch to a ConfigElement with a .name of "splat" that does not exist in the custom policy
        with self.assertRaises(ConfigElementNotFound):
            self.custom_policy.add_switch(config_element="splat", switch=Switch(name="name", value="value"))

    def test_add_switch___config_element_by_instance_not_found___raise_SwitchNotFound(self):
        # Try to add a switch to a ConfigElement instance that does not exist in the custom policy
        with self.assertRaises(ConfigElementNotFound):
            self.custom_policy.add_switch(config_element=ConfigElement(name="name"), switch=Switch(name="name", value="value"))

    def test_add_switch___config_element_by_str___switch_added(self):
        self.custom_policy.add_switch(
            config_element="pdfConfig",
            switch=Switch(name="embedded_files", value="sanitise")
        )

        # The embedded_files switch has been added, and has overwritten the previous switches value of "disallow"
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>sanitise</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """)
        )

    def test_add_switch___config_element_by_str_replace_False___switch_added(self):
        self.custom_policy.add_switch(
            config_element="pdfConfig",
            switch=Switch(name="embedded_files", value="sanitise"),
            replace=False
        )

        # The embedded_files switch has been appended, and has not overwritten the previous switch with a value of "disallow"
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                    <embedded_files>sanitise</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """)
        )

    def test_add_switch___config_element_by_instance___switch_added(self):
        config_element_pdfConfig = self.custom_policy.config_elements[1]
        self.custom_policy.add_switch(
            config_element=config_element_pdfConfig,
            switch=Switch(name="embedded_files", value="sanitise")
        )

        # The embedded_files switch has been added, and has overwritten the previous switches value of "disallow"
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>sanitise</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """)
        )

    def test_add_switch___config_element_by_instance_replace_False___switch_added(self):
        config_element_pdfConfig = self.custom_policy.config_elements[1]
        self.custom_policy.add_switch(
            config_element=config_element_pdfConfig,
            switch=Switch(name="embedded_files", value="sanitise"),
            replace=False
        )

        # The embedded_files switch has been appended, and has not overwritten the previous switch with a value of "disallow"
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                    <embedded_files>sanitise</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """)
        )

    def test_remove_config_element___config_element_by_str_config_element_not_found___raises_ConfigElementNotFound(self):
        # Try to remove a config element with a .name of "splat" that does not exist in the custom policy
        with self.assertRaises(ConfigElementNotFound):
            self.custom_policy.remove_config_element(config_element="splat")

    def test_remove_config_element___config_element_by_instance_config_element_not_found___raises_ConfigElementNotFound(self):
        # Try to remove a config element instance that does not exist in the custom policy
        with self.assertRaises(ConfigElementNotFound):
            self.custom_policy.remove_config_element(config_element=ConfigElement(name="splat"))

    def test_remove_config_element___config_element_by_str___config_element_removed(self):
        self.custom_policy.remove_config_element(config_element="pdfConfig")

        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
            </config>
            """)
        )

    def test_remove_config_element___config_element_by_str_with_duplicates___duplicate_config_element_removed(self):
        self.custom_policy_duplicates.remove_config_element(config_element="a")

        self.assertEqual(
            self.custom_policy_duplicates.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """)
        )

    def test_remove_config_element___config_element_by_instance___config_element_removed(self):
        config_element_pdfConfig = self.custom_policy.config_elements[1]
        self.custom_policy.remove_config_element(config_element=config_element_pdfConfig)

        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
            </config>
            """)
        )

    def test_add_config_element___replace_True___config_element_added(self):
        self.custom_policy.add_config_element(config_element=ConfigElement(name="pdfConfig"))

        # The empty pdfConfig config element has been added, and has overwritten the previous pdfConfig config element
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                </pdfConfig>
            </config>
            """)
        )

    def test_add_config_element___replace_False___config_element_added(self):
        self.custom_policy.add_config_element(config_element=ConfigElement(name="pdfConfig"), replace=False)

        # The empty pdfConfig config element has been appended, and has not overwritten the previous pdfConfig config element
        self.assertEqual(
            self.custom_policy.text,
            inspect.cleandoc("""
            <?xml version="1.0" encoding="utf-8"?>
            <config>
                <a>
                    <b>c</b>
                    <d f="g">e</d>
                </a>
                <pdfConfig>
                </pdfConfig>
                <pdfConfig>
                    <embedded_files>disallow</embedded_files>
                    <embedded_images>allow</embedded_images>
                </pdfConfig>
            </config>
            """)
        )

    def test_get_attributes(self):
        self.assertDictEqual(
            Policy.get_attributes({"one": "1", "@two": "2", "@three": "3"}),
            {"two": "2", "three": "3"}
        )

    def test_custom_editor_policy___switch_names_and_values_in_text(self):
        policy = glasswall.content_management.policies.Editor(
            config={
                "pdfConfig": {"a": "b"},
                "customConfig": {"customswitch": "customvalue"},
            }
        )

        self.assertTrue(policy.pdfConfig.a.value == "b")
        self.assertTrue(policy.customConfig.customswitch.value == "customvalue")

    def test_editor_custom_policy_with_attributes___attributes_set_and_not_included_as_switch(self):
        policy = glasswall.content_management.policies.Editor(
            config={
                "customConfig": {
                    "customswitch": "customvalue",
                    "@customattribute": "attributevalue",
                }
            }
        )

        # Attribute added correctly
        self.assertTrue(policy.customConfig.attributes.get("customattribute") == "attributevalue")

        # Attribute not added as switch
        self.assertTrue(not any(switch_name in policy.customConfig.get_switch_names() for switch_name in {"@customattribute", "customattribute"}))

    def test_editor_policy___getattr_missing_config_raises_AttributeError(self):
        policy = glasswall.content_management.policies.Editor(
            config={
                "customConfig": {
                    "customswitch": "customvalue",
                    "@customattribute": "attributevalue",
                }
            }
        )

        # getattr on missing config should raise AttributeError
        with self.assertRaises(AttributeError):
            policy.configNonexistant

        # getattr on missing switch should raise AttributeError
        with self.assertRaises(AttributeError):
            policy.customConfig.switchNonexistant

    def test_editor_policy___object_is_modifiable(self):
        policy = glasswall.content_management.policies.Editor(default="sanitise")

        # policy.pdfConfig.acroform.value should be sanitise
        self.assertTrue(policy.pdfConfig.acroform.value == "sanitise")

        # set policy.pdfConfig.acroform.value to allow
        policy.pdfConfig.acroform.value = "allow"

        # policy.pdfConfig.acroform.value should be allow
        self.assertTrue(policy.pdfConfig.acroform.value == "allow")

        # policy.pdfConfig.acroform.text should also be allow
        self.assertEqual(
            policy.pdfConfig.acroform.text,
            "<acroform>allow</acroform>"
        )

    def test_archive_manager_custom_policy___attributes_and_switches_customisable(self):
        policy = glasswall.content_management.policies.ArchiveManager(
            default="sanitise",
            default_archive_manager="process",
            config={
                "archiveConfig": {
                    "@recursionDepth": 1,
                    "@customAttribute": "customValue",
                    "bmp": "no_action",
                    "doc": "discard",
                }
            }
        )

        # archiveConfig has recursionDepth attribute with value 1
        self.assertTrue(policy.archiveConfig.attributes.get("recursionDepth") == 1)

        # archiveConfig has customAttribute attribute with value customValue
        self.assertTrue(policy.archiveConfig.attributes.get("customAttribute") == "customValue")

        # archiveConfig bmp switch is set to no_action
        self.assertTrue(policy.archiveConfig.bmp.value == "no_action")

        # archiveConfig doc switch is set to discard
        self.assertTrue(policy.archiveConfig.doc.value == "discard")

    def test_word_search_custom_policy___textList_order_maintained(self):
        policy = glasswall.content_management.policies.WordSearch(
            default="allow",
            config={
                "textSearchConfig": {
                    "textList": [
                        {"name": "textItem", "switches": [
                            {"name": "text", "value": "password"},
                            {"name": "textSetting", "@replacementChar": "*", "value": "redact"},
                        ]},
                        {"name": "textItem", "switches": [
                            {"name": "text", "value": "abc"},
                            {"name": "textSetting", "@replacementChar": "!", "value": "redact"},
                        ]},
                        {"name": "textItem", "switches": [
                            {"name": "text", "value": "xyz"},
                            {"name": "textSetting", "@replacementChar": "X", "value": "redact"},
                        ]},
                    ]
                }
            }
        )

        # Order should be unchanged
        self.assertEqual(
            policy.textSearchConfig.textList.text,
            inspect.cleandoc("""
            <textList>
                <textItem>
                    <text>password</text>
                    <textSetting replacementChar="*">redact</textSetting>
                </textItem>
                <textItem>
                    <text>abc</text>
                    <textSetting replacementChar="!">redact</textSetting>
                </textItem>
                <textItem>
                    <text>xyz</text>
                    <textSetting replacementChar="X">redact</textSetting>
                </textItem>
            </textList>
            """)
        )

    def test_word_search_custom_policy___attributes_customisable(self):
        policy = glasswall.content_management.policies.WordSearch(
            default="allow",
            config={
                "textSearchConfig": {
                    "textList": [
                        {"name": "textItem", "switches": [
                            {"name": "text", "value": "password"},
                            {"name": "textSetting", "@replacementChar": "*", "value": "redact", "@customAttribute": "customValue"},
                        ]},
                    ]
                }
            }
        )

        # textSetting attributes are customisable
        self.assertTrue(policy.textSearchConfig.textList.subelements[0].switches[1].attributes.get("customAttribute") == "customValue")

        # replacementChar attribute exists
        self.assertTrue(policy.textSearchConfig.textList.subelements[0].switches[1].attributes.get("replacementChar") == "*")


if __name__ == "__main__":
    unittest.main()





import ctypes as ct
import io
import os
import pathlib
import tempfile
from typing import Iterable, Union

from lxml import etree

import glasswall
from glasswall.config.logging import log


def as_bytes(file_: Union[bytes, bytearray, io.BytesIO]):
    """ Returns file_ as bytes.

    Args:
        file_ (Union[bytes, bytearray, io.BytesIO]): The file

    Returns:
        bytes

    Raises:
        TypeError: If file_ is not an instance of: bytes, bytearray, io.BytesIO
    """
    if isinstance(file_, bytes):
        return file_
    elif isinstance(file_, bytearray):
        return bytes(file_)
    elif isinstance(file_, io.BytesIO):
        return file_.read()
    else:
        raise TypeError(file_)


def as_io_BytesIO(file_: Union[bytes, bytearray]):
    """ Returns file_ as io.BytesIO object.

    Args:
        file_ (Union[bytes, bytearray]): The bytes or bytearray of the file

    Returns:
        io.BytesIO object

    Raises:
        TypeError: If file_ is not an instance of: bytes, bytearray, io.BytesIO
    """
    if isinstance(file_, bytes):
        return io.BytesIO(file_)
    elif isinstance(file_, bytearray):
        return io.BytesIO(bytes(file_))
    elif isinstance(file_, io.BytesIO):
        return file_
    else:
        raise TypeError(file_)


def as_snake_case(string):
    return ''.join(
        [
            '_' + char.lower()
            if char.isupper() else char
            for char in string
        ]
    ).lstrip('_')


def as_title(string):
    return ''.join(
        word.title()
        for word in string.split("_")
    )


def buffer_to_bytes(buffer: ct.c_void_p, buffer_length: ct.c_size_t):
    """ Convert ctypes buffer and buffer_length to bytes.

    Args:
        buffer (ct.c_void_p()): The file buffer.
        buffer_length (ct.c_size_t()): The file buffer length.

    Returns:
        bytes (bytes): The file as bytes.
    """

    file_buffer = (ct.c_byte * buffer_length.value)()
    ct.memmove(file_buffer, buffer.value, buffer_length.value)

    return bytes(file_buffer)


class CwdHandler:
    """ Changes the current working directory to new_cwd on __enter__, and back to previous cwd on __exit__.

    Args:
        new_cwd (str): The new current working directory to temporarily change to.
    """

    def __init__(self, new_cwd: str):
        self.new_cwd = new_cwd if os.path.isdir(new_cwd) else os.path.dirname(new_cwd)
        self.old_cwd = os.getcwd()

    def __enter__(self):
        os.chdir(self.new_cwd)

    def __exit__(self, type, value, traceback):
        os.chdir(self.old_cwd)


def delete_directory(directory: str, keep_folder: bool = False):
    """ Delete a directory and its contents.

    Args:
        directory (str): The directory path.
        keep_folder (bool, optional): Default False. If False, only delete contents.
    """
    if os.path.isdir(directory):
        # Delete all files in directory
        for file_ in list_file_paths(directory):
            os.remove(file_)

        # Delete all empty subdirectories
        delete_empty_subdirectories(directory)

        # Delete the directory
        if keep_folder is False:
            os.rmdir(directory)


def delete_empty_subdirectories(directory: str):
    """ Deletes all empty subdirectories of a given directory.

    Args:
        directory (str): The directory to delete subdirectories from.

    Returns:
        None
    """
    for root, dirs, _ in os.walk(directory, topdown=False):
        for dir_ in dirs:
            try:
                # Delete if empty
                os.rmdir(os.path.realpath(os.path.join(root, dir_)))
            except OSError:
                pass


def flatten_list(list_: Iterable):
    """ Returns a flattened list. [[1, 2], ["3"], (4, 5,), [6]] --> [1, 2, "3", 4, 5, 6] """
    return [
        item
        for sublist in list_
        for item in sublist
    ]


def get_libraries(directory: str, ignore_errors: bool = False):
    """ Recursively calls get_library on each library from glasswall.libraries.os_info on the given directory.

    Args:
        directory (str): The directory to search from.
        ignore_errors (bool, optional): Default False, prevents get_library raising FileNotFoundError when True.

    Returns:
        libraries (dict[str, str]): A dictionary of library names and their absolute file paths.
    """
    libraries = {}

    for library_name in glasswall.libraries.os_info[glasswall._OPERATING_SYSTEM].keys():
        try:
            libraries[library_name] = get_library(library_name, directory)
        except FileNotFoundError:
            if ignore_errors is True:
                continue
            raise

    return libraries


def get_library(library: str, directory: str):
    """ Returns a path to the specified library found from the current directory or any subdirectory. If multiple libraries exist, returns the file with the latest modified time.

    Args:
        library (str): The library to search for, ie: "rebuild", "word_search"
        directory (str): The directory to search from.

    Returns:
        library_file_path (str): The absolute file path to the library.

    Raises:
        KeyError: Unsupported OS or library name was not found in glasswall.libraries.os_info.
        FileNotFoundError: Library was not found.
    """
    library = as_snake_case(library)
    library_file_name = glasswall.libraries.os_info[glasswall._OPERATING_SYSTEM][library]["file_name"]

    p = pathlib.Path(directory)
    matching_files = list(p.rglob(library_file_name))

    if not matching_files:
        raise FileNotFoundError(f'Could not find file: "{library_file_name}" under directory: "{directory}"')

    library_file_path = str(max(matching_files, key=os.path.getctime).resolve())

    if len(matching_files) > 1:
        # warn that multiple libraries found, list library paths if there are <= 5
        if len(matching_files) <= 5:
            log.warning(f"Found {len(matching_files)} {library} libraries, but expected only one:\n{chr(10).join(str(item) for item in matching_files)}\nLatest library: {library_file_path}")
        else:
            log.warning(f"Found {len(matching_files)} {library} libraries, but expected only one.\nLatest library: {library_file_path}")

    return library_file_path


def list_file_paths(directory: str, recursive: bool = True, absolute: bool = True, followlinks: bool = True):
    """ Returns a list of paths to files in a directory.

    Args:
        directory (str): The directory to list files from.
        recursive (bool, optional): Default True. Include subdirectories.
        absolute (bool, optional): Default True. Return paths as absolute paths. If False, returns relative paths.
        followlinks (bool, optional): Default True. Follow symbolic links if True.

    Returns:
        files (list): A list of file paths.
    """
    if not os.path.isdir(directory):
        raise NotADirectoryError(directory)

    if recursive:
        files = [
            os.path.normpath(os.path.join(root, file_))
            for root, dirs, files in os.walk(directory)
            for file_ in files
        ]
    else:
        files = [
            os.path.normpath(os.path.join(directory, file_))
            for file_ in os.listdir(directory)
            if os.path.isfile(os.path.join(directory, file_))
        ]

    if followlinks:
        files = [
            os.readlink(file_)
            if os.path.islink(file_)
            else file_
            for file_ in files
        ]

    if absolute:
        files = [
            os.path.abspath(file_)
            for file_ in files
        ]
    else:
        files = [
            os.path.relpath(file_, directory)
            for file_ in files
        ]

    return files


def list_subdirectory_paths(directory: str, recursive: bool = False, absolute: bool = True):
    """ Returns a list of paths to subdirectories in a directory.

    Args:
        directory (str): The directory to list subdirectories from.
        recursive (bool, optional): Default False. Include subdirectories of subdirectories.
        absolute (bool, optional): Default True. Return paths as absolute paths. If False, returns relative paths.

    Returns:
        subdirectories (list): A list of subdirectory paths.
    """
    subdirectories = [f.path for f in os.scandir(directory) if f.is_dir()]

    if recursive:
        for subdirectory in subdirectories:
            subdirectories.extend(list_subdirectory_paths(subdirectory, recursive=True))

    if absolute:
        subdirectories = [os.path.abspath(path) for path in subdirectories]
    else:
        subdirectories = [os.path.relpath(path, directory) for path in subdirectories]

    return subdirectories


def load_dependencies(dependencies: list, ignore_errors: bool = False):
    """ Calls ctypes.cdll.LoadLibrary on each file path in `dependencies`.

    Args:
        dependencies (list): A list of absolute file paths of library dependencies.
        ignore_errors (bool, optional): Default False, avoid raising exceptions from ct.cdll.LoadLibrary if ignore_errors is True.

    Returns:
        missing_dependencies (list): A list of missing dependencies, or an empty list.
    """
    missing_dependencies = [dependency for dependency in dependencies if not os.path.isfile(dependency)]

    for dependency in dependencies:
        # Try to load dependencies that exist
        if dependency not in missing_dependencies:
            try:
                ct.cdll.LoadLibrary(dependency)
            except Exception:
                if ignore_errors:
                    pass
                else:
                    raise

    return missing_dependencies


class TempDirectoryPath:
    """ Gives a path to a uniquely named temporary directory that does not currently exist on __enter__, deletes the directory if it exists on __exit__.

    Args:
        delete (bool, optional): Default True. Delete the temporary directory on __exit__
    """

    def __init__(self, delete: bool = True):
        # Validate args
        if not isinstance(delete, bool):
            raise TypeError(delete)

        self.temp_directory = None
        self.delete = delete

        while self.temp_directory is None or os.path.isdir(self.temp_directory):
            self.temp_directory = os.path.join(glasswall._TEMPDIR, next(tempfile._get_candidate_names()))

        # Create temp directory
        os.makedirs(self.temp_directory, exist_ok=True)

    def __enter__(self):
        return self.temp_directory

    def __exit__(self, type, value, traceback):
        if self.delete:
            # Delete temp directory and all of its contents
            if os.path.isdir(self.temp_directory):
                delete_directory(self.temp_directory)


class TempFilePath:
    """ Gives a path to a uniquely named temporary file that does not currently exist on __enter__, deletes the file if it exists on __exit__.

    Args:
        directory (Union[str, None], optional): The directory to create a temporary file in.
        delete (bool, optional): Default True. Delete the temporary file on on __exit__
    """

    def __init__(self, directory: Union[str, None] = None, delete: bool = True):
        # Validate args
        if not isinstance(directory, (str, type(None))):
            raise TypeError(directory)
        if isinstance(directory, str) and not os.path.isdir(directory):
            raise NotADirectoryError(directory)
        if not isinstance(delete, bool):
            raise TypeError(delete)

        self.temp_file = None
        self.directory = directory or tempfile.gettempdir()
        self.delete = delete

        while self.temp_file is None or os.path.isfile(self.temp_file):
            self.temp_file = os.path.join(self.directory, next(tempfile._get_candidate_names()))

        # Create temp directory if it does not exist
        os.makedirs(os.path.dirname(self.temp_file), exist_ok=True)

    def __enter__(self):
        return self.temp_file

    def __exit__(self, type, value, traceback):
        if self.delete:
            if os.path.isfile(self.temp_file):
                os.remove(self.temp_file)


# NOTE typehint as string due to no "from __future__ import annotations" support on python 3.6 on ubuntu-16.04 / centos7
def validate_xml(xml: Union[str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"]):
    """ Attempts to parse the xml provided, returning the xml as string. Raises ValueError if the xml cannot be parsed.

    Args:
        xml (Union[str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy]): The xml string, or file path, bytes, or ContentManagementPolicy instance to parse.

    Returns:
        xml_string (str): A string representation of the xml.

    Raises:
        ValueError: if the xml cannot be parsed.
        TypeError: if the type of arg "xml" is invalid
    """
    try:
        # Get tree from file
        if isinstance(xml, str) and os.path.isfile(xml):
            tree = etree.parse(xml)

        # Get tree from xml string
        elif isinstance(xml, str):
            xml = xml.encode("utf-8")
            tree = etree.fromstring(xml)

        # Get tree from bytes, bytearray, io.BytesIO
        elif isinstance(xml, (bytes, bytearray, io.BytesIO)):
            # Convert bytes, bytearray to io.BytesIO
            if isinstance(xml, (bytes, bytearray)):
                xml = as_io_BytesIO(xml)
            tree = etree.parse(xml)

        # Get tree from ContentManagementPolicy instance
        elif isinstance(xml, glasswall.content_management.policies.Policy):
            xml = xml.text.encode("utf-8")
            tree = etree.fromstring(xml)

        else:
            raise TypeError(xml)

    except etree.XMLSyntaxError:
        raise ValueError(xml)

    # # convert tree to string and include xml declaration header utf8
    etree.indent(tree, space=" " * 4)
    xml_string = etree.tostring(tree, encoding="utf-8", xml_declaration=True, pretty_print=True).decode()

    return xml_string


def xml_as_dict(xml):
    """ Converts a simple single-level xml into a dictionary.

    Args:
        xml (Union[str, bytes, bytearray, io.BytesIO]): The xml string, or file path, or bytes to parse.

    Returns:
        dict_ (dict): A dictionary of element tag : text
    """
    # Convert xml to string
    xml_string = validate_xml(xml)

    # Get root
    root = etree.fromstring(xml_string.encode())

    dict_ = {
        element.tag: element.text
        for element in root
    }

    # Sort for ease of viewing logs
    dict_ = {k: v for k, v in sorted(dict_.items())}

    return dict_





import glasswall
from glasswall.content_management.policies.policy import Policy


class ArchiveManager(Policy):
    """ A content management policy for ArchiveManager.

    Args:
        default (str): The default action to be performed. (allow, disallow, sanitise)
        default_archive_manager (str): The default action to be performed for archiveConfig. (no_action, discard, process)
        config (dict): Additional configuration settings passed to the ConfigElement with the same name as the key.

    Example:
        ArchiveManager(
            default="allow",
            default_archive_manager="process",
            config={
                "pdfConfig": {"embeddedImages": "disallow"},
                "wordConfig": {"embeddedImages": "disallow"},
                "archiveConfig": {
                    "@recursionDepth": "100",
                    "jpeg": "discard"
                }
            }
        )
    """

    def __init__(self, default: str = "sanitise", default_archive_manager: str = "process", config: dict = {}):
        self.default = default
        self.default_archive_manager = default_archive_manager
        self.default_config_elements = [
            glasswall.content_management.config_elements.archiveConfig,
            glasswall.content_management.config_elements.pdfConfig,
            glasswall.content_management.config_elements.pptConfig,
            glasswall.content_management.config_elements.tiffConfig,
            glasswall.content_management.config_elements.wordConfig,
            glasswall.content_management.config_elements.xlsConfig,
        ]
        self.config = config or {}

        super().__init__(
            default=self.default,
            default_archive_manager=default_archive_manager,
            default_config_elements=self.default_config_elements,
            config=self.config,
        )





import unittest

from glasswall.content_management.errors.switches import RestrictedValue
from glasswall.content_management.switches.switch import Switch


class TestSwitch(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        self.switch_embedded_images_allow = Switch(
            name="embedded_images",
            value="allow",
            restrict_values=["allow", "disallow", "sanitise"]
        )

        self.switch_custom_with_attributes = Switch(
            name="custom",
            value="sanitise",
            attributes={"a": "1", "b": "2"}
        )

    def tearDown(self):
        pass

    def test_text___no_attributes___text_matches_expected(self):
        self.assertEqual(
            self.switch_embedded_images_allow.text,
            '<embedded_images>allow</embedded_images>'
        )

    def test_text___with_attributes___text_matches_expected(self):
        # text writes attributes in alphabetical order, lowercase (key, value)
        self.assertEqual(
            self.switch_custom_with_attributes.text,
            '<custom a="1" b="2">sanitise</custom>'
        )

    def test_value___change_value_to_disallow___text_shows_value_as_disallow(self):
        self.switch_custom_with_attributes.value = "disallow"
        self.assertEqual(
            self.switch_custom_with_attributes.text,
            '<custom a="1" b="2">disallow</custom>'
        )

    def test_restrict_values___change_value_within_restrict_values___text_shows_value_as_disallow(self):
        self.switch_embedded_images_allow.value = "disallow"
        self.assertEqual(
            self.switch_embedded_images_allow.text,
            '<embedded_images>disallow</embedded_images>'
        )

    def test_restrict_values___change_value_outside_restrict_values___raises_RestrictedValue(self):
        with self.assertRaises(RestrictedValue):
            self.switch_embedded_images_allow.value = "splat"

    def test_repr___matches_expected(self):
        self.assertEqual(
            repr(self.switch_embedded_images_allow),
            'Switch("embedded_images", "allow")'
        )

    def test_lt___sort_list_of_switches___ordered_by_name_then_value(self):
        L = [
            Switch(
                name="b",
                value="allow",
            ),
            Switch(
                name="c",
                value="allow",
            ),
            Switch(
                name="c",
                value="sanitise",
            ),
            Switch(
                name="a",
                value="disallow",
            ),
            Switch(
                name="B",
                value="disallow",
            ),
        ]

        L.sort()

        self.assertEqual(
            str(L),
            '[Switch("a", "disallow"), Switch("b", "allow"), Switch("B", "disallow"), Switch("c", "allow"), Switch("c", "sanitise")]'
        )


if __name__ == "__main__":
    unittest.main()





class SecurityTaggingError(Exception):
    """ Base class for all SecurityTagging errors. """
    pass


class SecurityTaggingSuccess():
    """ Base class for all SecurityTagging successes. """
    pass





import os
import platform
import tempfile

__version__ = "0.2.3"

_OPERATING_SYSTEM = platform.system()
_PYTHON_VERSION = platform.python_version()
_ROOT = os.path.dirname(__file__)
_TEMPDIR = os.path.join(os.environ.get("AGENT_TEMPDIRECTORY", tempfile.gettempdir()), "glasswall")

from glasswall import config, content_management, determine_file_type, utils
from glasswall.libraries.archive_manager.archive_manager import ArchiveManager
from glasswall.libraries.editor.editor import Editor
from glasswall.libraries.rebuild.rebuild import Rebuild
from glasswall.libraries.security_tagging.security_tagging import SecurityTagging
from glasswall.libraries.word_search.word_search import WordSearch


class GwReturnObj:
    """ A Glasswall return object. """

    def __init__(self, **kwargs):
        [setattr(self, k, v) for k, v in kwargs.items()]





from glasswall.content_management.config_elements.config_element import ConfigElement
from glasswall.content_management.config_elements.textList import textList
from glasswall.content_management.policies.policy import Policy
from glasswall.content_management.switches.switch import Switch


class textSearchConfig(ConfigElement):
    """ A textSearchConfig ConfigElement.

    textSearchConfig(
        libVersion="core2",
        textList_subelements=[
            {"name": "textItem", "switches": [
                {"name": "text", "value": "generic"},
                {"name": "textSetting", "@replacementChar": "*", "value": "redact"},
            ]}
        ]
    )
    """

    def __init__(self, attributes: dict = {}, textList_subelements: list = [], **kwargs):
        self.name = self.__class__.__name__
        self.attributes = attributes or {}
        self.attributes = {
            **{
                "libVersion": kwargs.get("libVersion", "core2"),
            },
            **self.attributes,
        }
        subelements = []
        for textList_dict in textList_subelements:
            switch_list = []
            for switch_dict in textList_dict.get("switches", []):
                # Construct switch
                switch = Switch(
                    **Policy.get_switches(switch_dict),
                    attributes=Policy.get_attributes(switch_dict)
                )

                # Append switch to switch_list
                switch_list.append(switch)

            # Overwrite textList_dict switches with switch_list objects
            textList_dict["switches"] = switch_list

            # Construct config_element
            config_element = ConfigElement(**textList_dict)

            # Append constructed config_element to subelements
            subelements.append(config_element)

        self.subelements = [
            textList(subelements=subelements)
        ]

        super().__init__(name=self.name, attributes=self.attributes, subelements=self.subelements)





from glasswall.libraries.rebuild.classes import RebuildError


class UnknownErrorCode(RebuildError):
    """ Unknown error code. """
    pass


# Statuses from sdk.rebuild\src\code\dll.gwfile\dll.gwfilestatus.h
class Error(RebuildError):
    """ Rebuild error code 0. This value indicates that the document was non-conformant in some way, but any requested output files were written. """
    pass


class SuccessDocumentWriteFailure(RebuildError):
    """ Rebuild error code -1. This value indicates that the document was managed successfully, but a failure occured when writing the managed version of the document to file. """
    pass


class SuccessAnalysisWriteFailure(RebuildError):
    """ Rebuild error code -2. This value indicates that the document was analysed successfully, but a failure occured when writing the analysis of the document to file. """
    pass


class ErrorAnalysisWriteFailure(RebuildError):
    """ Rebuild error code -3. This value indicates that the document was non-conformant in some way, and a failure occured when writing the analysis of the document to file. """
    pass


class SuccessReportWriteFailure(RebuildError):
    """ Rebuild error code -4. This value indicates that the document was processed successfully, but that a failure occured when writing the processing report to file. """
    pass


class SuccessDocumentReportWriteFailure(RebuildError):
    """ Rebuild error code -5. This value indicates that the document was managed successfully, but a failure occured when writing both the managed version of the document and the processing report to file. """
    pass


class ErrorReportWriteFailure(RebuildError):
    """ Rebuild error code -6. This value indicates that the document was non-conformant in some way,  and that a failure occured when writing the processing report to file. """
    pass


class SuccessAnalysisReportWriteFailure(RebuildError):
    """ Rebuild error code -7. This value indicates that the document was analysed successfully, but a failure occured when writing both the analysis of the document and the processing report to file. """
    pass


class ErrorAnalysisReportWriteFailure(RebuildError):
    """ Rebuild error code -8. This value indicates that the document was non-conformant in some way, but a failure occured when writing both the analysis of the document and the processing report to file. """
    pass


class InternalError(RebuildError):
    """ Rebuild error code -9. This value indicates an uncategorised error """
    pass


class SuccessDocumentAnalysisReportWriteFailure(RebuildError):
    """ Rebuild error code -10. This value indicates that the document was analysed successfully, but failures occured when writing the document, the analysis of the document and the processing report to file (AMP mode). """
    pass


class SuccessDocumentAnalysisWriteFailure(RebuildError):
    """ Rebuild error code -11. This value indicates that the document was analysed successfully, but failures occured when writing the document and the analysis of the document to file (AMP mode). """
    pass


class SuccessExportWriteFailure(RebuildError):
    """ Rebuild error code -12. This value indicates that the document was exported successfully, but failures occured when writing the archive package. (Export mode) """
    pass


class ErrorExportWriteFailure(RebuildError):
    """ Rebuild error code -13. This value indicates that the document was non-conformant in some way, and failures occured when writing the archive package. (Export mode) """
    pass


error_codes = {
    0: Error,
    -1: SuccessDocumentWriteFailure,
    -2: SuccessAnalysisWriteFailure,
    -3: ErrorAnalysisWriteFailure,
    -4: SuccessReportWriteFailure,
    -5: SuccessDocumentReportWriteFailure,
    -6: ErrorReportWriteFailure,
    -7: SuccessAnalysisReportWriteFailure,
    -8: ErrorAnalysisReportWriteFailure,
    -9: InternalError,
    -10: SuccessDocumentAnalysisReportWriteFailure,
    -11: SuccessDocumentAnalysisWriteFailure,
    -12: SuccessExportWriteFailure,
    -13: ErrorExportWriteFailure,
}





import ctypes as ct
import os

import glasswall
from glasswall import utils


class Library:
    """ A Glasswall library. """

    def __init__(self, library_path: str):
        self.library_path = library_path

    def load_library(self, library_path: str):
        if not os.path.isfile(library_path):
            if os.path.isdir(library_path):
                library_path = utils.get_library(self.__class__.__name__, library_path)
            else:
                raise FileNotFoundError(library_path)

        self.library_path = library_path

        # Preload dependencies to avoid "OSError: ...: cannot open shared object file: No such file or directory"
        dependencies = [
            os.path.join(os.path.dirname(self.library_path), dependency)
            for dependency in glasswall.libraries.os_info[glasswall._OPERATING_SYSTEM][utils.as_snake_case(self.__class__.__name__)]["dependencies"]
        ]
        missing_dependencies = utils.load_dependencies(dependencies, ignore_errors=True)

        with utils.CwdHandler(new_cwd=self.library_path):
            try:
                # Try to load library
                return ct.cdll.LoadLibrary(self.library_path)
            except OSError as e:
                # If library fails to load and there are missing dependencies, list them
                if missing_dependencies:
                    raise FileNotFoundError(f"Unable to load {self.__class__.__name__}. Below dependencies are missing in directory: {os.path.dirname(self.library_path)}\n{', '.join(missing_dependencies)}") from e
                raise





import ctypes as ct
import io
import os
from contextlib import contextmanager
from typing import Union

import glasswall
from glasswall import determine_file_type as dft
from glasswall import utils
from glasswall.config.logging import log
from glasswall.libraries.editor import errors, successes
from glasswall.libraries.library import Library


class Editor(Library):
    """ A high level Python wrapper for Glasswall Editor / Core2. """

    def __init__(self, library_path: str):
        super().__init__(library_path)
        self.library = self.load_library(os.path.abspath(library_path))

        # Validate killswitch has not activated
        self.validate_license()

        log.info(f"Loaded Glasswall {self.__class__.__name__} version {self.version()} from {self.library_path}")

    def validate_license(self):
        """ Validates the license of the library by attempting to call protect_file on a known supported file.

        Raises:
            LicenseExpired: If the license has expired.
            EditorError: If the license could not be validated.
        """
        # Call protect file on a known good bitmap to see if license has expired
        try:
            self.protect_file(
                input_file=b"BM:\x00\x00\x00\x00\x00\x00\x006\x00\x00\x00(\x00\x00\x00\x01\x00\x00\x00\x01\x00\x00\x00\x01\x00\x18\x00\x00\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff\xff\xff\x00",
                raise_unsupported=True
            )
            log.debug(f"{self.__class__.__name__} license validated successfully.")
        except errors.EditorError:
            log.warning(f"{self.__class__.__name__} license validation failed.")
            raise

    def version(self):
        """ Returns the Glasswall library version.

        Returns:
            version (str): The Glasswall library version.
        """
        # API function declaration
        self.library.GW2LibVersion.restype = ct.c_char_p

        # API call
        version = self.library.GW2LibVersion()

        # Convert to Python string
        version = ct.string_at(version).decode()

        return version

    def open_session(self):
        """ Open a new Glasswall session.

        Returns:
            session (int): An incrementing integer repsenting the current session.
        """
        # API call
        session = self.library.GW2OpenSession()

        log.debug(f"\n\tsession: {session}")

        return session

    def close_session(self, session: int):
        """ Close the Glasswall session. All resources allocated by the session will be destroyed.

        Args:
            session (int): The session to close.

        Returns:
            None
        """
        if not isinstance(session, int):
            raise TypeError(session)

        # API function declaration
        self.library.GW2CloseSession.argtypes = [ct.c_size_t]

        # Variable initialisation
        ct_session = ct.c_size_t(session)

        # API call
        status = self.library.GW2CloseSession(ct_session)

        if status not in successes.success_codes:
            log.warning(f"\n\tsession: {session}\n\tstatus: {status}")
        else:
            log.debug(f"\n\tsession: {session}\n\tstatus: {status}")

        return status

    @contextmanager
    def new_session(self):
        """ Context manager. Opens a new session on entry and closes the session on exit. """
        try:
            session = self.open_session()
            yield session
        finally:
            self.close_session(session)

    def run_session(self, session):
        """ Runs the Glasswall session and begins processing of a file.

        Args:
            session (int): The session to run.

        Returns:
            status (int): The status of the function call.
        """
        # API function declaration
        self.library.GW2RunSession.argtypes = [ct.c_size_t]

        # Variable initialisation
        ct_session = ct.c_size_t(session)

        # API call
        status = self.library.GW2RunSession(ct_session)

        if status not in successes.success_codes:
            log.warning(f"\n\tsession: {session}\n\tstatus: {status}")
        else:
            log.debug(f"\n\tsession: {session}\n\tstatus: {status}")

        return status

    def determine_file_type(self, input_file: Union[str, bytes, bytearray, io.BytesIO], as_string: bool = False):
        """ Returns an int representing the file type / file format of a file.

        Args:
            input_file (Union[str, bytes, bytearray, io.BytesIO]): The input file, can be a local path.
            as_string (bool, optional): Return file type as string, eg: "bmp" instead of: 29. Defaults to False.

        Returns:
            file_type (Union[int, str]): The file format.
        """
        if isinstance(input_file, str):
            if not os.path.isfile(input_file):
                raise FileNotFoundError(input_file)

            # convert to ct.c_char_p of bytes
            ct_input_file = ct.c_char_p(input_file.encode("utf-8"))

            # API call
            file_type = self.library.GW2DetermineFileTypeFromFile(ct_input_file)

        elif isinstance(input_file, (bytes, bytearray, io.BytesIO)):
            # convert to bytes
            bytes_input_file = utils.as_bytes(input_file)

            # ctypes conversion
            ct_buffer = ct.c_char_p(bytes_input_file)
            ct_butter_length = ct.c_size_t(len(bytes_input_file))

            # API call
            file_type = self.library.GW2DetermineFileTypeFromMemory(
                ct_buffer,
                ct_butter_length
            )

        else:
            raise TypeError(input_file)

        file_type_as_string = dft.file_type_int_to_str(file_type)
        input_file_repr = f"{type(input_file)} length {len(input_file)}" if isinstance(input_file, (bytes, bytearray,)) else input_file.__sizeof__() if isinstance(input_file, io.BytesIO) else input_file

        if not dft.is_success(file_type):
            log.warning(f"\n\tfile_type: {file_type}\n\tfile_type_as_string: {file_type_as_string}\n\tinput_file: {input_file_repr}")
            raise dft.int_class_map.get(file_type, dft.errors.UnknownErrorCode)(file_type)
        else:
            log.debug(f"\n\tfile_type: {file_type}\n\tfile_type_as_string: {file_type_as_string}\n\tinput_file: {input_file_repr}")

        if as_string:
            return file_type_as_string

        return file_type

    def get_content_management_policy(self, session: int):
        """ Returns the content management configuration for a given session.

        Args:
            session (int): The current session.

        Returns:
            xml_string (str): The XML string of the current content management configuration.
        """
        # NOTE GW2GetPolicySettings is current not implemented in editor

        # set xml_string as loaded default config
        xml_string = glasswall.content_management.policies.Editor(default="sanitise").text,

        # log.debug(f"xml_string:\n{xml_string}")

        return xml_string

        # # API function declaration
        # self.library.GW2GetPolicySettings.argtypes = [
        #     ct.c_size_t,
        #     ct.c_void_p,
        # ]

        # # Variable initialisation
        # ct_session = ct.c_size_t(session)
        # ct_buffer = ct.c_void_p()
        # ct_butter_length = ct.c_size_t()
        # # ct_file_format = ct.c_int(file_format)

        # # API Call
        # status = self.library.GW2GetPolicySettings(
        #     ct_session,
        #     ct.byref(ct_buffer),
        #     ct.byref(ct_butter_length)
        # )

        # print("GW2GetPolicySettings status:", status)

        # file_bytes = utils.buffer_to_bytes(
        #     ct_buffer,
        #     ct_butter_length,
        # )

        # return file_bytes

    def set_content_management_policy(self, session: int, input_file: Union[None, str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"] = None, file_format=0):
        """ Sets the content management policy configuration. If input_file is None then default settings (sanitise) are applied.

        Args:
            session (int): The current session.
            input_file (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): Default None (sanitise). The content management policy to apply.
            file_format (int): The file format of the content management policy. 0 is xml.

        Returns:
            status (int): The result of the Glasswall API call.
        """
        # Validate type
        if not isinstance(session, int):
            raise TypeError(session)
        if not isinstance(input_file, (type(None), str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy)):
            raise TypeError(input_file)
        if not isinstance(file_format, int):
            raise TypeError(file_format)

        # Set input_file to default if input_file is None
        if input_file is None:
            input_file = glasswall.content_management.policies.Editor(default="sanitise")

        # Validate xml content is parsable
        utils.validate_xml(input_file)

        gw_return_object = glasswall.GwReturnObj()

        # From file
        if isinstance(input_file, str) and os.path.isfile(input_file):
            # API function declaration
            self.library.GW2RegisterPoliciesFile.argtypes = [
                ct.c_size_t,
                ct.c_char_p,
                ct.c_int,
            ]

            # Variable initialisation
            gw_return_object.ct_session = ct.c_size_t(session)
            gw_return_object.ct_input_file = ct.c_char_p(input_file.encode("utf-8"))
            gw_return_object.ct_file_format = ct.c_int(file_format)

            gw_return_object.status = self.library.GW2RegisterPoliciesFile(
                gw_return_object.ct_session,
                gw_return_object.ct_input_file,
                gw_return_object.ct_file_format
            )

        # From memory
        elif isinstance(input_file, (str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy)):
            # Convert bytearray, io.BytesIO to bytes
            if isinstance(input_file, (bytearray, io.BytesIO)):
                input_file = utils.as_bytes(input_file)
            # Convert string xml or Policy to bytes
            if isinstance(input_file, (str, glasswall.content_management.policies.Policy)):
                input_file = input_file.encode("utf-8")

            # API function declaration
            self.library.GW2RegisterPoliciesMemory.argtype = [
                ct.c_size_t,
                ct.c_char_p,
                ct.c_int
            ]

            # Variable initialisation
            gw_return_object.ct_session = ct.c_size_t(session)
            gw_return_object.ct_buffer = ct.c_char_p(input_file)
            gw_return_object.ct_buffer_length = ct.c_size_t(len(input_file))
            gw_return_object.ct_file_format = ct.c_int(file_format)

            # API Call
            gw_return_object.status = self.library.GW2RegisterPoliciesMemory(
                gw_return_object.ct_session,
                gw_return_object.ct_buffer,
                gw_return_object.ct_buffer_length,
                gw_return_object.ct_file_format
            )

        if gw_return_object.status not in successes.success_codes:
            log.warning(f"\n\tsession: {session}\n\tstatus: {gw_return_object.status}")
            raise errors.error_codes.get(gw_return_object.status, errors.UnknownErrorCode)(gw_return_object.status)
        else:
            log.debug(f"\n\tsession: {session}\n\tstatus: {gw_return_object.status}")

        return gw_return_object

    def register_input(self, session: int, input_file: Union[str, bytes, bytearray, io.BytesIO]):
        """ Register an input file or bytes for the given session.

        Args:
            session (int): The current session.
            input_file (Union[str, bytes, bytearray, io.BytesIO]): The input file path or bytes.

        Returns:
            status (int): The result of the Glasswall API call.
        """

        if not isinstance(input_file, (str, bytes, bytearray, io.BytesIO,)):
            raise TypeError(input_file)

        if isinstance(input_file, str):
            if not os.path.isfile(input_file):
                raise FileNotFoundError(input_file)

            # API function declaration
            self.library.GW2RegisterInputFile.argtypes = [
                ct.c_size_t,
                ct.c_char_p
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            ct_input_file = ct.c_char_p(input_file.encode("utf-8"))

            # API call
            status = self.library.GW2RegisterInputFile(
                ct_session,
                ct_input_file
            )

        elif isinstance(input_file, (bytes, bytearray, io.BytesIO,)):
            # Convert bytearray and io.BytesIO to bytes
            input_file = utils.as_bytes(input_file)

            # API function declaration
            self.library.GW2RegisterInputMemory.argtypes = [
                ct.c_size_t,
                ct.c_char_p,
                ct.c_size_t,
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            ct_buffer = ct.c_char_p(input_file)
            ct_buffer_length = ct.c_size_t(len(input_file))

            # API call
            status = self.library.GW2RegisterInputMemory(
                ct_session,
                ct_buffer,
                ct_buffer_length
            )

        if status not in successes.success_codes:
            log.warning(f"\n\tsession: {session}\n\tstatus: {status}")
            raise errors.error_codes.get(status, errors.UnknownErrorCode)(status)
        else:
            log.debug(f"\n\tsession: {session}\n\tstatus: {status}")

        return status

    def register_output(self, session, output_file: Union[None, str] = None):
        """ Register an output file for the given session. If output_file is None the file will be returned as 'buffer' and 'buffer_length' attributes.

        Args:
            session (int): The current session.
            output_file (Union[None, str], optional): If specified, during run session the file will be written to output_file, otherwise the file will be written to the glasswall.GwReturnObj 'buffer' and 'buffer_length' attributes.

        Returns:
            gw_return_object (glasswall.GwReturnObj): A GwReturnObj instance with the attribute 'status' indicating the result of the function call. If output_file is None (memory mode), 'buffer', and 'buffer_length' are included containing the file content and file size.
        """

        if not isinstance(output_file, (type(None), str)):
            raise TypeError(output_file)

        gw_return_object = glasswall.GwReturnObj()

        if isinstance(output_file, str):
            # API function declaration
            self.library.GW2RegisterOutFile.argtypes = [
                ct.c_size_t,
                ct.c_char_p
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            ct_output_file = ct.c_char_p(output_file.encode("utf-8"))

            # API call
            gw_return_object.status = self.library.GW2RegisterOutFile(
                ct_session,
                ct_output_file
            )

        else:
            # API function declaration
            self.library.GW2RegisterOutputMemory.argtypes = [
                ct.c_size_t,
                ct.POINTER(ct.c_void_p),
                ct.POINTER(ct.c_size_t)
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            gw_return_object.buffer = ct.c_void_p()
            gw_return_object.buffer_length = ct.c_size_t(0)

            # API call
            gw_return_object.status = self.library.GW2RegisterOutputMemory(
                ct_session,
                ct.byref(gw_return_object.buffer),
                ct.byref(gw_return_object.buffer_length)
            )

        if gw_return_object.status not in successes.success_codes:
            log.warning(f"\n\tsession: {session}\n\tstatus: {gw_return_object.status}")
            raise errors.error_codes.get(gw_return_object.status, errors.UnknownErrorCode)(gw_return_object.status)
        else:
            log.debug(f"\n\tsession: {session}\n\tstatus: {gw_return_object.status}")

        return gw_return_object

    def register_analysis(self, session: int, output_file: Union[None, str] = None):
        """ Registers an analysis file for the given session. The analysis file will be created during the session's run_session call.

        Args:
            session (int): The session number.
            output_file (Union[None, str], optional): Default None. The file path where the analysis will be written. None returns the analysis as bytes.

        Returns:
            gw_return_object (glasswall.GwReturnObj): A GwReturnObj instance with the attribute 'status' indicating the result of the function call. If output_file is None (memory mode), 'buffer', and 'buffer_length' are included containing the file content and file size.
        """

        if not isinstance(output_file, (type(None), str)):
            raise TypeError(output_file)

        if isinstance(output_file, str):
            output_file = os.path.abspath(output_file)

            # API function declaration
            self.library.GW2RegisterAnalysisFile.argtypes = [
                ct.c_size_t,
                ct.c_char_p,
                ct.c_int,
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            ct_output_file = ct.c_char_p(output_file.encode("utf-8"))
            analysis_file_format = ct.c_int()
            gw_return_object = glasswall.GwReturnObj()

            # API call
            status = self.library.GW2RegisterAnalysisFile(
                ct_session,
                ct_output_file,
                analysis_file_format
            )

        elif isinstance(output_file, type(None)):
            # API function declaration
            self.library.GW2RegisterAnalysisMemory.argtypes = [
                ct.c_size_t,
                ct.POINTER(ct.c_void_p),
                ct.POINTER(ct.c_size_t)
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            gw_return_object = glasswall.GwReturnObj()
            gw_return_object.buffer = ct.c_void_p()
            gw_return_object.buffer_length = ct.c_size_t()

            # API call
            status = self.library.GW2RegisterAnalysisMemory(
                ct_session,
                ct.byref(gw_return_object.buffer),
                ct.byref(gw_return_object.buffer_length)
            )

        if status not in successes.success_codes:
            log.warning(f"\n\tsession: {session}\n\tstatus: {status}")
            raise errors.error_codes.get(status, errors.UnknownErrorCode)(status)
        else:
            log.debug(f"\n\tsession: {session}\n\tstatus: {status}")

        gw_return_object.status = status

        return gw_return_object

    def protect_file(self, input_file: Union[str, bytes, bytearray, io.BytesIO], output_file: Union[None, str] = None, content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"] = None, raise_unsupported: bool = True):
        """ Protects a file using the current content management configuration, returning the file bytes. The protected file is written to output_file if it is provided.

        Args:
            input_file (Union[str, bytes, bytearray, io.BytesIO]): The input file path or bytes.
            output_file (Union[None, str], optional): The output file path where the protected file will be written.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): The content management policy to apply to the session.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            file_bytes (bytes): The protected file bytes.
        """
        # Validate arg types
        if not isinstance(input_file, (str, bytes, bytearray, io.BytesIO)):
            raise TypeError(input_file)
        if not isinstance(output_file, (type(None), str)):
            raise TypeError(output_file)
        if not isinstance(content_management_policy, (type(None), str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy)):
            raise TypeError(content_management_policy)
        if not isinstance(raise_unsupported, bool):
            raise TypeError(raise_unsupported)

        # Convert string path arguments to absolute paths
        if isinstance(input_file, str):
            if not os.path.isfile(input_file):
                raise FileNotFoundError(input_file)
            input_file = os.path.abspath(input_file)
        if isinstance(output_file, str):
            output_file = os.path.abspath(output_file)
            # make directories that do not exist
            os.makedirs(os.path.dirname(output_file), exist_ok=True)
        if isinstance(content_management_policy, str) and os.path.isfile(content_management_policy):
            content_management_policy = os.path.abspath(content_management_policy)

        # Convert memory inputs to bytes
        if isinstance(input_file, (bytes, bytearray, io.BytesIO)):
            input_file = utils.as_bytes(input_file)

        # Check that file type is supported
        try:
            self.determine_file_type(input_file=input_file)
        except dft.errors.FileTypeEnumError:
            if raise_unsupported:
                raise
            else:
                return None

        with utils.CwdHandler(self.library_path):
            with self.new_session() as session:
                content_management_policy = self.set_content_management_policy(session, content_management_policy)
                register_input = self.register_input(session, input_file)
                register_output = self.register_output(session, output_file=output_file)
                status = self.run_session(session)
                # Ensure memory allocated is not garbage collected until after run_session
                content_management_policy, register_input, register_output

                if status not in successes.success_codes:
                    if raise_unsupported:
                        raise errors.error_codes.get(status, errors.UnknownErrorCode)(status)
                    else:
                        file_bytes = None
                else:
                    # Get file bytes
                    if isinstance(output_file, str):
                        # File to file and memory to file, Glasswall wrote to a file, read it to get the file bytes
                        if not os.path.isfile(output_file):
                            log.warning(f"Glasswall returned success code: {status} but no output file was found: {output_file}")
                            file_bytes = None
                        else:
                            with open(output_file, "rb") as f:
                                file_bytes = f.read()
                    else:
                        # File to memory and memory to memory, Glasswall wrote to a buffer, convert it to bytes
                        file_bytes = utils.buffer_to_bytes(
                            register_output.buffer,
                            register_output.buffer_length
                        )

                return file_bytes

    def protect_directory(self, input_directory: str, output_directory: Union[None, str], content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"] = None, raise_unsupported: bool = True):
        """ Recursively processes all files in a directory in protect mode using the given content management policy.
        The protected files are written to output_directory maintaining the same directory structure as input_directory.

        Args:
            input_directory (str): The input directory containing files to protect.
            output_directory (Union[None, str]): The output directory where the protected file will be written, or None to not write files.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): Default None (sanitise). The content management policy to apply.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            protected_files_dict (dict): A dictionary of file paths relative to input_directory, and file bytes.
        """
        protected_files_dict = {}
        # Call protect_file on each file in input_directory to output_directory
        for input_file in utils.list_file_paths(input_directory):
            relative_path = os.path.relpath(input_file, input_directory)
            output_file = None if output_directory is None else os.path.join(os.path.abspath(output_directory), relative_path)

            protected_bytes = self.protect_file(
                input_file=input_file,
                output_file=output_file,
                raise_unsupported=raise_unsupported,
                content_management_policy=content_management_policy,
            )

            protected_files_dict[relative_path] = protected_bytes

        return protected_files_dict

    def analyse_file(self, input_file: Union[str, bytes, bytearray, io.BytesIO], output_file: Union[None, str] = None, content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"] = None, raise_unsupported: bool = True):
        """ Analyses a file, returning the analysis bytes. The analysis is written to output_file if it is provided.

        Args:
            input_file (Union[str, bytes, bytearray, io.BytesIO]): The input file path or bytes.
            output_file (Union[None, str], optional): The output file path where the analysis file will be written.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): The content management policy to apply to the session.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            file_bytes (bytes): The analysis file bytes.
        """
        # Validate arg types
        if not isinstance(input_file, (str, bytes, bytearray, io.BytesIO)):
            raise TypeError(input_file)
        if not isinstance(output_file, (type(None), str)):
            raise TypeError(output_file)
        if not isinstance(content_management_policy, (type(None), str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy)):
            raise TypeError(content_management_policy)
        if not isinstance(raise_unsupported, bool):
            raise TypeError(raise_unsupported)

        # Check that file type is supported
        try:
            self.determine_file_type(input_file=input_file)
        except dft.errors.FileTypeEnumError:
            if raise_unsupported:
                raise
            else:
                return None

        # Convert string path arguments to absolute paths
        if isinstance(input_file, str):
            if not os.path.isfile(input_file):
                raise FileNotFoundError(input_file)
            input_file = os.path.abspath(input_file)
        if isinstance(output_file, str):
            output_file = os.path.abspath(output_file)
            # make directories that do not exist
            os.makedirs(os.path.dirname(output_file), exist_ok=True)
        if isinstance(content_management_policy, str) and os.path.isfile(content_management_policy):
            content_management_policy = os.path.abspath(content_management_policy)

        # Convert memory inputs to bytes
        if isinstance(input_file, (bytes, bytearray, io.BytesIO)):
            input_file = utils.as_bytes(input_file)

        with utils.CwdHandler(self.library_path):
            with self.new_session() as session:
                content_management_policy = self.set_content_management_policy(session, content_management_policy)
                register_input = self.register_input(session, input_file)
                register_analysis = self.register_analysis(session, output_file)
                status = self.run_session(session)
                # Ensure memory allocated is not garbage collected until after run_session
                content_management_policy, register_input, register_analysis

                if status not in successes.success_codes:
                    if raise_unsupported:
                        raise errors.error_codes.get(status, errors.UnknownErrorCode)(status)
                    else:
                        file_bytes = None
                else:
                    # Get file bytes
                    if isinstance(output_file, str):
                        # File to file and memory to file, Glasswall wrote to a file, read it to get the file bytes
                        if not os.path.isfile(output_file):
                            log.warning(f"Glasswall returned success code: {status} but no output file was found: {output_file}")
                            file_bytes = None
                        else:
                            with open(output_file, "rb") as f:
                                file_bytes = f.read()
                    else:
                        # File to memory and memory to memory, Glasswall wrote to a buffer, convert it to bytes
                        file_bytes = utils.buffer_to_bytes(
                            register_analysis.buffer,
                            register_analysis.buffer_length
                        )

                return file_bytes

    def analyse_directory(self, input_directory: str, output_directory: Union[None, str], content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"] = None, raise_unsupported: bool = True):
        """ Analyses all files in a directory and its subdirectories. The analysis files are written to output_directory maintaining the same directory structure as input_directory.

        Args:
            input_directory (str): The input directory containing files to analyse.
            output_directory (Union[None, str]): The output directory where the analysis files will be written, or None to not write files.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): Default None (sanitise). The content management policy to apply.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            analysis_files_dict (dict): A dictionary of file paths relative to input_directory, and file bytes.
        """
        analysis_files_dict = {}
        # Call analyse_file on each file in input_directory to output_directory
        for input_file in utils.list_file_paths(input_directory):
            relative_path = os.path.relpath(input_file, input_directory) + ".xml"
            output_file = None if output_directory is None else os.path.join(os.path.abspath(output_directory), relative_path)

            analysis_bytes = self.analyse_file(
                input_file=input_file,
                output_file=output_file,
                raise_unsupported=raise_unsupported,
                content_management_policy=content_management_policy,
            )

            analysis_files_dict[relative_path] = analysis_bytes

        return analysis_files_dict

    def register_export(self, session: int, output_file: Union[None, str] = None):
        """ Registers a file to be exported for the given session. The export file will be created during the session's run_session call.

        Args:
            session (int): The session number.
            output_file (Union[None, str], optional): Default None. The file path where the export will be written. None returns the export as bytes.

        Returns:
            gw_return_object (glasswall.GwReturnObj): A GwReturnObj instance with the attribute 'status' indicating the result of the function call. If output_file is None (memory mode), 'buffer', and 'buffer_length' are included containing the file content and file size.
        """

        if not isinstance(output_file, (type(None), str)):
            raise TypeError(output_file)

        if isinstance(output_file, str):
            output_file = os.path.abspath(output_file)

            # API function declaration
            self.library.GW2RegisterExportFile.argtypes = [
                ct.c_size_t,
                ct.c_char_p
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            ct_output_file = ct.c_char_p(output_file.encode("utf-8"))
            gw_return_object = glasswall.GwReturnObj()

            # API Call
            status = self.library.GW2RegisterExportFile(
                ct_session,
                ct_output_file
            )

        elif isinstance(output_file, type(None)):
            # API function declaration
            self.library.GW2RegisterExportMemory.argtypes = [
                ct.c_size_t,
                ct.POINTER(ct.c_void_p),
                ct.POINTER(ct.c_size_t)
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            gw_return_object = glasswall.GwReturnObj()
            gw_return_object.buffer = ct.c_void_p()
            gw_return_object.buffer_length = ct.c_size_t()

            # API call
            status = self.library.GW2RegisterExportMemory(
                ct_session,
                ct.byref(gw_return_object.buffer),
                ct.byref(gw_return_object.buffer_length)
            )

        if status not in successes.success_codes:
            log.warning(f"\n\tsession: {session}\n\tstatus: {status}")
            raise errors.error_codes.get(status, errors.UnknownErrorCode)(status)
        else:
            log.debug(f"\n\tsession: {session}\n\tstatus: {status}")

        gw_return_object.status = status

        return gw_return_object

    def export_file(self, input_file: Union[str, bytes, bytearray, io.BytesIO], output_file: Union[None, str] = None, content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"] = None, raise_unsupported: bool = True):
        """ Export a file, returning the .zip file bytes. The .zip file is written to output_file if it is provided.

        Args:
            input_file (Union[str, bytes, bytearray, io.BytesIO]): The input file path or bytes.
            output_file (Union[None, str], optional): The output file path where the .zip file will be written.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): The content management policy to apply to the session.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            file_bytes (bytes): The exported .zip file.
        """
        # Validate arg types
        if not isinstance(input_file, (str, bytes, bytearray, io.BytesIO)):
            raise TypeError(input_file)
        if not isinstance(output_file, (type(None), str)):
            raise TypeError(output_file)
        if not isinstance(content_management_policy, (type(None), str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy)):
            raise TypeError(content_management_policy)
        if not isinstance(raise_unsupported, bool):
            raise TypeError(raise_unsupported)

        # Check that file type is supported
        try:
            self.determine_file_type(input_file=input_file)
        except dft.errors.FileTypeEnumError:
            if raise_unsupported:
                raise
            else:
                return None

        # Convert string path arguments to absolute paths
        if isinstance(input_file, str):
            if not os.path.isfile(input_file):
                raise FileNotFoundError(input_file)
            input_file = os.path.abspath(input_file)
        if isinstance(output_file, str):
            output_file = os.path.abspath(output_file)
            # make directories that do not exist
            os.makedirs(os.path.dirname(output_file), exist_ok=True)
        if isinstance(content_management_policy, str) and os.path.isfile(content_management_policy):
            content_management_policy = os.path.abspath(content_management_policy)

        # Convert memory inputs to bytes
        if isinstance(input_file, (bytes, bytearray, io.BytesIO)):
            input_file = utils.as_bytes(input_file)

        with utils.CwdHandler(self.library_path):
            with self.new_session() as session:
                content_management_policy = self.set_content_management_policy(session, content_management_policy)
                register_input = self.register_input(session, input_file)
                register_export = self.register_export(session, output_file)
                status = self.run_session(session)
                # Ensure memory allocated is not garbage collected until after run_session
                content_management_policy, register_input, register_export

                if status not in successes.success_codes:
                    log.warning(f"\n\tsession: {session}\n\tstatus: {status}")
                    if raise_unsupported:
                        raise errors.error_codes.get(status, errors.UnknownErrorCode)(status)
                    else:
                        file_bytes = None
                else:
                    # Get file bytes
                    if isinstance(output_file, str):
                        # File to file and memory to file, Glasswall wrote to a file, read it to get the file bytes
                        if not os.path.isfile(output_file):
                            log.warning(f"Glasswall returned success code: {status} but no output file was found: {output_file}")
                            file_bytes = None
                        else:
                            with open(output_file, "rb") as f:
                                file_bytes = f.read()
                    else:
                        # File to memory and memory to memory, Glasswall wrote to a buffer, convert it to bytes
                        file_bytes = utils.buffer_to_bytes(
                            register_export.buffer,
                            register_export.buffer_length
                        )

                return file_bytes

    def export_directory(self, input_directory: str, output_directory: Union[None, str], content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"] = None, raise_unsupported: bool = True):
        """ Exports all files in a directory and its subdirectories. The export files are written to output_directory maintaining the same directory structure as input_directory.

        Args:
            input_directory (str): The input directory containing files to export.
            output_directory (Union[None, str]): The output directory where the export files will be written, or None to not write files.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): Default None (sanitise). The content management policy to apply.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            export_files_dict (dict): A dictionary of file paths relative to input_directory, and file bytes.
        """
        export_files_dict = {}
        # Call export_file on each file in input_directory to output_directory
        for input_file in utils.list_file_paths(input_directory):
            relative_path = os.path.relpath(input_file, input_directory) + ".zip"
            output_file = None if output_directory is None else os.path.join(os.path.abspath(output_directory), relative_path)

            export_bytes = self.export_file(
                input_file=input_file,
                output_file=output_file,
                raise_unsupported=raise_unsupported,
                content_management_policy=content_management_policy,
            )

            export_files_dict[relative_path] = export_bytes

        return export_files_dict

    def register_import(self, session: int, input_file: Union[str, bytes, bytearray, io.BytesIO]):
        """ Registers a .zip file to be imported for the given session. The constructed file will be created during the session's run_session call.

        Args:
            session (int): The session number.
            input_file (Union[str, bytes, bytearray, io.BytesIO]): The input file path or bytes.

        Returns:
            gw_return_object (glasswall.GwReturnObj): A GwReturnObj instance with the attribute 'status' indicating the result of the function call. If output_file is None (memory mode), 'buffer', and 'buffer_length' are included containing the file content and file size.
        """
        if not isinstance(input_file, (str, bytes, bytearray, io.BytesIO,)):
            raise TypeError(input_file)

        if isinstance(input_file, str):
            if not os.path.isfile(input_file):
                raise FileNotFoundError(input_file)

        gw_return_object = glasswall.GwReturnObj()

        if isinstance(input_file, str):
            input_file = os.path.abspath(input_file)

            # API function declaration
            self.library.GW2RegisterImportFile.argtypes = [
                ct.c_size_t,
                ct.c_char_p
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            ct_input_file = ct.c_char_p(input_file.encode("utf-8"))

            # API Call
            gw_return_object.status = self.library.GW2RegisterImportFile(
                ct_session,
                ct_input_file
            )

        elif isinstance(input_file, (bytes, bytearray, io.BytesIO,)):
            # Convert bytearray and io.BytesIO to bytes
            input_file = utils.as_bytes(input_file)

            # API function declaration
            self.library.GW2RegisterImportMemory.argtypes = [
                ct.c_size_t,
                ct.c_void_p,
                ct.c_size_t
            ]

            # Variable initialisation
            ct_session = ct.c_size_t(session)
            gw_return_object.buffer = ct.c_char_p(input_file)
            gw_return_object.buffer_length = ct.c_size_t(len(input_file))

            # API call
            gw_return_object.status = self.library.GW2RegisterImportMemory(
                ct_session,
                gw_return_object.buffer,
                gw_return_object.buffer_length
            )

        if gw_return_object.status not in successes.success_codes:
            log.warning(f"\n\tsession: {session}\n\tstatus: {gw_return_object.status}")
            raise errors.error_codes.get(gw_return_object.status, errors.UnknownErrorCode)(gw_return_object.status)
        else:
            log.debug(f"\n\tsession: {session}\n\tstatus: {gw_return_object.status}")

        return gw_return_object

    def import_file(self, input_file: Union[str, bytes, bytearray, io.BytesIO], output_file: Union[None, str] = None, content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"] = None, raise_unsupported: bool = True):
        """ Import a .zip file, constructs a file from the .zip file and returns the file bytes. The file is written to output_file if it is provided.

        Args:
            input_file (Union[str, bytes, bytearray, io.BytesIO]): The .zip input file path or bytes.
            output_file (Union[None, str], optional): The output file path where the constructed file will be written.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): The content management policy to apply to the session.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            file_bytes (bytes): The imported file bytes.
        """
        # Validate arg types
        if not isinstance(input_file, (str, bytes, bytearray, io.BytesIO)):
            raise TypeError(input_file)
        if not isinstance(output_file, (type(None), str)):
            raise TypeError(output_file)
        if not isinstance(content_management_policy, (type(None), str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy)):
            raise TypeError(content_management_policy)
        if not isinstance(raise_unsupported, bool):
            raise TypeError(raise_unsupported)

        # Check that file type is supported
        try:
            self.determine_file_type(input_file=input_file)
        except dft.errors.FileTypeEnumError:
            if raise_unsupported:
                raise
            else:
                return None

        # Convert string path arguments to absolute paths
        if isinstance(input_file, str):
            if not os.path.isfile(input_file):
                raise FileNotFoundError(input_file)
            input_file = os.path.abspath(input_file)
        if isinstance(output_file, str):
            output_file = os.path.abspath(output_file)
            # make directories that do not exist
            os.makedirs(os.path.dirname(output_file), exist_ok=True)
        if isinstance(content_management_policy, str) and os.path.isfile(content_management_policy):
            content_management_policy = os.path.abspath(content_management_policy)

        # Convert memory inputs to bytes
        if isinstance(input_file, (bytes, bytearray, io.BytesIO)):
            input_file = utils.as_bytes(input_file)

        with utils.CwdHandler(self.library_path):
            with self.new_session() as session:
                content_management_policy = self.set_content_management_policy(session, content_management_policy)
                register_import = self.register_import(session, input_file)
                register_output = self.register_output(session, output_file)
                status = self.run_session(session)
                # Ensure memory allocated is not garbage collected until after run_session
                content_management_policy, register_import, register_output

                if status not in successes.success_codes:
                    log.warning(f"\n\tsession: {session}\n\tstatus: {status}")
                    if raise_unsupported:
                        raise errors.error_codes.get(status, errors.UnknownErrorCode)(status)
                    else:
                        file_bytes = None
                else:
                    # Get file bytes
                    if isinstance(output_file, str):
                        # File to file and memory to file, Glasswall wrote to a file, read it to get the file bytes
                        if not os.path.isfile(output_file):
                            log.warning(f"Glasswall returned success code: {status} but no output file was found: {output_file}")
                            file_bytes = None
                        else:
                            with open(output_file, "rb") as f:
                                file_bytes = f.read()
                    else:
                        # File to memory and memory to memory, Glasswall wrote to a buffer, convert it to bytes
                        file_bytes = utils.buffer_to_bytes(
                            register_output.buffer,
                            register_output.buffer_length
                        )

                return file_bytes

    def import_directory(self, input_directory: str, output_directory: Union[None, str], content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO, "glasswall.content_management.policies.Policy"] = None, raise_unsupported: bool = True):
        """ Imports all files in a directory and its subdirectories. Files are expected as .zip but this is not forced.
        The constructed files are written to output_directory maintaining the same directory structure as input_directory.

        Args:
            input_directory (str): The input directory containing files to import.
            output_directory (Union[None, str]): The output directory where the constructed files will be written, or None to not write files.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): Default None (sanitise). The content management policy to apply.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            import_files_dict (dict): A dictionary of file paths relative to input_directory, and file bytes.
        """
        import_files_dict = {}
        # Call import_file on each file in input_directory to output_directory
        for input_file in utils.list_file_paths(input_directory):
            relative_path = os.path.relpath(input_file, input_directory)
            # Remove .zip extension from relative_path
            relative_path = os.path.splitext(relative_path)[0]
            output_file = None if output_directory is None else os.path.join(os.path.abspath(output_directory), relative_path)

            import_bytes = self.import_file(
                input_file=input_file,
                output_file=output_file,
                raise_unsupported=raise_unsupported,
                content_management_policy=content_management_policy,
            )

            import_files_dict[relative_path] = import_bytes

        return import_files_dict





import ctypes as ct
import functools
import io
import os
from typing import Union, Optional

import glasswall
from glasswall import utils
from glasswall.config.logging import log
from glasswall.libraries.archive_manager import errors, successes
from glasswall.libraries.library import Library


class ArchiveManager(Library):
    """ A high level Python wrapper for Glasswall Archive Manager. """
    class Decorators:
        @classmethod
        def release_after(cls, function):
            """ Call function followed by self.release() """
            @functools.wraps(function)
            def wrapper(self, *args, **kwargs):
                result = function(self, *args, **kwargs)
                self.release()
                return result
            return wrapper

    def __init__(self, library_path):
        super().__init__(library_path)
        self.library = self.load_library(os.path.abspath(library_path))

        log.info(f"Loaded Glasswall {self.__class__.__name__} version {self.version()} from {self.library_path}")

    def version(self):
        """ Returns the Glasswall library version.

        Returns:
            version (str): The Glasswall library version.
        """
        # API function declaration
        self.library.GwArchiveVersion.restype = ct.c_char_p

        # API call
        version = self.library.GwArchiveVersion()

        # Convert to Python string
        version = ct.string_at(version).decode()

        return version

    def release(self):
        """ Releases any resources held by the Glasswall Archive Manager library. """
        self.library.GwArchiveDone()

    @Decorators.release_after
    def analyse_archive(self, input_file: Union[str, bytes, bytearray, io.BytesIO], output_file: Union[None, str] = None, output_report: Union[None, str] = None, content_management_policy: Union[type(None), str, bytes, bytearray, io.BytesIO] = None, raise_unsupported: bool = True):
        """ Extracts the input_file archive and processes each file within the archive using the Glasswall engine. Repackages all files regenerated by the Glasswall engine into a new archive, optionally writing the new archive and report to the paths specified by output_file and output_report.

        Args:
            input_file (Union[str, bytes, bytearray, io.BytesIO]): The archive file path or bytes.
            output_file (Union[None, str], optional): Default None. If str, write the new archive to the output_file path.
            output_report (Union[None, str], optional): Default None. If str, write a report to the output_file path.
            content_management_policy (Union[str, bytes, bytearray, io.BytesIO)], optional): The content_management_policy file path or bytes.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            gw_return_object (glasswall.GwReturnObj): An instance of class glasswall.GwReturnObj containing attributes: "status" (int), "output_file" (bytes), "output_report" (bytes)
        """
        # Validate arg types
        if not isinstance(input_file, (str, bytes, bytearray, io.BytesIO)):
            raise TypeError(input_file)
        if not isinstance(output_file, (type(None), str)):
            raise TypeError(output_file)
        if not isinstance(output_report, (type(None), str)):
            raise TypeError(output_report)
        if not isinstance(content_management_policy, (type(None), str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy)):
            raise TypeError(content_management_policy)

        # Convert string path arguments to absolute paths
        if isinstance(output_file, str):
            output_file = os.path.abspath(output_file)

        if isinstance(output_report, str):
            output_report = os.path.abspath(output_report)

        # Convert inputs to bytes
        if isinstance(input_file, str):
            if not os.path.isfile(input_file):
                raise FileNotFoundError(input_file)
            with open(input_file, "rb") as f:
                input_file_bytes = f.read()
        elif isinstance(input_file, (bytearray, io.BytesIO)):
            input_file_bytes = utils.as_bytes(input_file)

        if isinstance(content_management_policy, str) and os.path.isfile(content_management_policy):
            with open(content_management_policy, "rb") as f:
                content_management_policy = f.read()
        elif isinstance(content_management_policy, type(None)):
            # Load default
            content_management_policy = glasswall.content_management.policies.ArchiveManager(default="sanitise", default_archive_manager="process")
        content_management_policy = utils.validate_xml(content_management_policy)

        # API function declaration
        self.library.GwFileAnalysisArchive.argtypes = [
            ct.c_void_p,
            ct.c_size_t,
            ct.POINTER(ct.c_void_p),
            ct.POINTER(ct.c_size_t),
            ct.POINTER(ct.c_void_p),
            ct.POINTER(ct.c_size_t),
            ct.c_char_p
        ]

        # Variable initialisation
        input_buffer_bytearray = bytearray(input_file_bytes)

        ct_input_buffer = (ct.c_ubyte * len(input_buffer_bytearray)).from_buffer(input_buffer_bytearray)  # void *inputBuffer
        ct_input_buffer_length = ct.c_size_t(len(input_file_bytes))  # size_t inputBufferLength
        ct_output_buffer = ct.c_void_p()  # void **outputFileBuffer
        ct_output_buffer_length = ct.c_size_t()  # size_t *outputFileBufferLength
        ct_output_report_buffer = ct.c_void_p()  # void **outputAnalysisReportBuffer
        ct_output_report_buffer_length = ct.c_size_t()  # size_t *outputAnalysisReportBufferLength
        ct_content_management_policy = ct.c_char_p(content_management_policy.encode())  # const char *xmlConfigString
        gw_return_object = glasswall.GwReturnObj()

        with utils.CwdHandler(new_cwd=self.library_path):
            # API call
            gw_return_object.status = self.library.GwFileAnalysisArchive(
                ct.byref(ct_input_buffer),
                ct_input_buffer_length,
                ct.byref(ct_output_buffer),
                ct.byref(ct_output_buffer_length),
                ct.byref(ct_output_report_buffer),
                ct.byref(ct_output_report_buffer_length),
                ct_content_management_policy
            )

        if gw_return_object.status not in successes.success_codes:
            if raise_unsupported:
                raise errors.error_codes.get(gw_return_object.status, errors.UnknownErrorCode)(gw_return_object.status)

        gw_return_object.output_file = utils.buffer_to_bytes(
            ct_output_buffer,
            ct_output_buffer_length
        )
        gw_return_object.output_report = utils.buffer_to_bytes(
            ct_output_report_buffer,
            ct_output_report_buffer_length
        )

        # Write output file
        if gw_return_object.output_file:
            if isinstance(output_file, str):
                os.makedirs(os.path.dirname(output_file), exist_ok=True)
                with open(output_file, "wb") as f:
                    f.write(gw_return_object.output_file)

        # Write output report
        if gw_return_object.output_report:
            if isinstance(output_report, str):
                os.makedirs(os.path.dirname(output_report), exist_ok=True)
                with open(output_report, "wb") as f:
                    f.write(gw_return_object.output_report)

        return gw_return_object

    def analyse_directory(self, input_directory: str, output_directory: str, content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO] = None, raise_unsupported: bool = True):
        """ Calls analyse_archive on each file in input_directory using the given content management configuration. The resulting archives and analysis reports are written to output_directory maintaining the same directory structure as input_directory.

        Args:
            input_directory (str): The input directory containing archives to analyse.
            output_directory (str): The output directory where the new archives and reports will be written.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO], optional): The content management policy .xml to use.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            None
        """
        for relative_path in utils.list_file_paths(input_directory, absolute=False):
            # construct absolute paths
            input_file = os.path.abspath(os.path.join(input_directory, relative_path))
            output_file = os.path.abspath(os.path.join(output_directory, relative_path))

            # call analyse_archive on each file in input_directory
            self.analyse_archive(
                input_file=input_file,
                output_file=output_file,
                output_report=output_file + ".xml",
                content_management_policy=content_management_policy,
                raise_unsupported=raise_unsupported,
            )

    @Decorators.release_after
    def protect_archive(self, input_file: Union[str, bytes, bytearray, io.BytesIO], output_file: Union[None, str] = None, output_report: Union[None, str] = None, content_management_policy: Union[type(None), str, bytes, bytearray, io.BytesIO] = None, raise_unsupported: bool = True):
        """ Extracts the input_file archive and processes each file within the archive using the Glasswall engine. Repackages all files regenerated by the Glasswall engine into a new archive, optionally writing the new archive and report to the paths specified by output_file and output_report.

        Args:
            input_file (Union[str, bytes, bytearray, io.BytesIO]): The archive file path or bytes.
            output_file (Union[None, str], optional): Default None. If str, write the new archive to the output_file path.
            output_report (Union[None, str], optional): Default None. If str, write a report to the output_file path.
            content_management_policy (Union[str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy)], optional): The content_management_policy file path or bytes.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            gw_return_object (glasswall.GwReturnObj): An instance of class glasswall.GwReturnObj containing attributes: "status" (int), "output_file" (bytes), "output_report" (bytes)
        """
        # Validate arg types
        if not isinstance(input_file, (str, bytes, bytearray, io.BytesIO)):
            raise TypeError(input_file)
        if not isinstance(output_file, (type(None), str)):
            raise TypeError(output_file)
        if not isinstance(output_report, (type(None), str)):
            raise TypeError(output_report)
        if not isinstance(content_management_policy, (type(None), str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy)):
            raise TypeError(content_management_policy)

        # Convert string path arguments to absolute paths
        if isinstance(output_file, str):
            output_file = os.path.abspath(output_file)

        if isinstance(output_report, str):
            output_report = os.path.abspath(output_report)

        # Convert inputs to bytes
        if isinstance(input_file, str):
            if not os.path.isfile(input_file):
                raise FileNotFoundError(input_file)
            with open(input_file, "rb") as f:
                input_file_bytes = f.read()
        elif isinstance(input_file, (bytearray, io.BytesIO)):
            input_file_bytes = utils.as_bytes(input_file)

        if isinstance(content_management_policy, str) and os.path.isfile(content_management_policy):
            with open(content_management_policy, "rb") as f:
                content_management_policy = f.read()
        elif isinstance(content_management_policy, type(None)):
            # Load default
            content_management_policy = glasswall.content_management.policies.ArchiveManager(default="sanitise", default_archive_manager="process")
        content_management_policy = utils.validate_xml(content_management_policy)

        # API function declaration
        self.library.GwFileAnalysisArchive.argtypes = [
            ct.c_void_p,
            ct.c_size_t,
            ct.POINTER(ct.c_void_p),
            ct.POINTER(ct.c_size_t),
            ct.POINTER(ct.c_void_p),
            ct.POINTER(ct.c_size_t),
            ct.c_char_p
        ]

        # Variable initialisation
        input_buffer_bytearray = bytearray(input_file_bytes)

        ct_input_buffer = (ct.c_ubyte * len(input_buffer_bytearray)).from_buffer(input_buffer_bytearray)  # void *inputBuffer
        ct_input_buffer_length = ct.c_size_t(len(input_file_bytes))  # size_t inputBufferLength
        ct_output_buffer = ct.c_void_p()  # void **outputFileBuffer
        ct_output_buffer_length = ct.c_size_t()  # size_t *outputFileBufferLength
        ct_output_report_buffer = ct.c_void_p()  # void **outputAnalysisReportBuffer
        ct_output_report_buffer_length = ct.c_size_t()  # size_t *outputAnalysisReportBufferLength
        ct_content_management_policy = ct.c_char_p(content_management_policy.encode())  # const char *xmlConfigString
        gw_return_object = glasswall.GwReturnObj()

        with utils.CwdHandler(new_cwd=self.library_path):
            # API call
            gw_return_object.status = self.library.GwFileProtectAndReportArchive(
                ct.byref(ct_input_buffer),
                ct_input_buffer_length,
                ct.byref(ct_output_buffer),
                ct.byref(ct_output_buffer_length),
                ct.byref(ct_output_report_buffer),
                ct.byref(ct_output_report_buffer_length),
                ct_content_management_policy
            )

        if gw_return_object.status not in successes.success_codes:
            if raise_unsupported:
                raise errors.error_codes.get(gw_return_object.status, errors.UnknownErrorCode)(gw_return_object.status)

        gw_return_object.output_file = utils.buffer_to_bytes(
            ct_output_buffer,
            ct_output_buffer_length
        )
        gw_return_object.output_report = utils.buffer_to_bytes(
            ct_output_report_buffer,
            ct_output_report_buffer_length
        )

        # Write output file
        if gw_return_object.output_file:
            if isinstance(output_file, str):
                os.makedirs(os.path.dirname(output_file), exist_ok=True)
                with open(output_file, "wb") as f:
                    f.write(gw_return_object.output_file)

        # Write output report
        if gw_return_object.output_report:
            if isinstance(output_report, str):
                os.makedirs(os.path.dirname(output_report), exist_ok=True)
                with open(output_report, "wb") as f:
                    f.write(gw_return_object.output_report)

        return gw_return_object

    def protect_directory(self, input_directory: str, output_directory: Union[None, str], output_report_directory: Union[None, str] = None, content_management_policy: Union[None, str, bytes, bytearray, io.BytesIO] = None, raise_unsupported: bool = True):
        """ Calls protect_archive on each file in input_directory using the given content management configuration. The resulting archives are written to output_directory maintaining the same directory structure as input_directory.

        Args:
            input_directory (str): The input directory containing archives to protect.
            output_directory (Union[None, str], optional): Default None. If str, the output directory where the new archives will be written.
            output_report_directory (Union[None, str], optional): Default None. If str, the output directory where xml reports for each archive will be written.
            content_management_policy (Union[None, str, bytes, bytearray, io.BytesIO, glasswall.content_management.policies.Policy], optional): The content management policy .xml to use.
            raise_unsupported (bool, optional): Default True. Raise exceptions when Glasswall encounters an error. Fail silently if False.

        Returns:
            protected_archives_dict (dict): A dictionary of file paths relative to input_directory, and glasswall.GwReturnObj containing `output_file` and output_report` attributes of type: bytes.
        """
        protected_archives_dict = {}
        # Call protect_archive on each file in input_directory to output_directory
        for input_file in utils.list_file_paths(input_directory):
            relative_path = os.path.relpath(input_file, input_directory)
            output_file = None if output_directory is None else os.path.join(os.path.abspath(output_directory), relative_path)
            output_report = None if output_report_directory is None else os.path.join(os.path.abspath(output_report_directory), relative_path + ".xml")

            gw_return_object = self.protect_archive(
                input_file=input_file,
                output_file=output_file,
                output_report=output_report,
                content_management_policy=content_management_policy,
                raise_unsupported=raise_unsupported,
            )

            protected_archives_dict[relative_path] = gw_return_object

        return protected_archives_dict

    def file_to_file_unpack(self, input_file: str, output_directory: str, file_type: Optional[str] = None, raise_unsupported: bool = True):
        # Validate arg types
        if not isinstance(input_file, str):
            raise TypeError(input_file)
        elif not os.path.isfile(input_file):
            raise FileNotFoundError(input_file)
        if not isinstance(output_directory, str):
            raise TypeError(output_directory)
        if not file_type:
            file_type = ArchiveManager.get_archive_type(input_file)

        # API function declaration
        self.library.GwFileToFileUnpack.argtypes = [
            ct.c_char_p,
            ct.c_char_p,
            ct.c_char_p,
        ]

        # Variable initialisation
        gw_return_object = glasswall.GwReturnObj()
        gw_return_object.ct_input_file = ct.c_char_p(input_file.encode())  # const char* inputFilePath
        gw_return_object.ct_output_directory = ct.c_char_p(output_directory.encode())  # const char* outputDirPath
        gw_return_object.ct_file_type = ct.c_char_p(file_type.encode())  # const char *fileType

        with utils.CwdHandler(new_cwd=self.library_path):
            # API call
            gw_return_object.status = self.library.GwFileToFileUnpack(
                gw_return_object.ct_input_file,
                gw_return_object.ct_output_directory,
                gw_return_object.ct_file_type,
            )

        if gw_return_object.status not in successes.success_codes:
            if raise_unsupported:
                raise errors.error_codes.get(gw_return_object.status, errors.UnknownErrorCode)(gw_return_object.status)

        return gw_return_object

    @staticmethod
    def get_archive_type(file_path: str):
        """ Returns the archive type of a file path """
        file_type = os.path.splitext(file_path)[-1].replace(".", "", 1)
        if not file_type:
            raise Exception(f"File type not indentifiable: {file_path}")
        return file_type

    @staticmethod
    def is_supported_archive_type(archive_type: str):
        """ Returns True if the file_path is an archive that is supported. """
        return archive_type in {
            "7z",
            "bz2",
            "gz",
            "rar",
            "tar",
            "zip",
        }

    @staticmethod
    def filter_archives(file_paths: list):
        """ Filters a list of files, returning supported archives only. """
        return [
            file_path
            for file_path in file_paths
            if ArchiveManager.is_supported_archive_type(ArchiveManager.get_archive_type(file_path))
        ]

    @staticmethod
    def get_archives(directory: str):
        """ Returns a list of supported archives in a directory and all subdirectories. """
        return ArchiveManager.filter_archives(glasswall.utils.list_file_paths(directory))

    def recursively_unpack(self, input_file: str, output_directory: str, file_type: Optional[str] = None, raise_unsupported: bool = True, delete_origin: bool = False):
        # Maintain directory structure of archives, converting archives to directories
        archive_basename = os.path.splitext(os.path.basename(input_file))[0]
        output_directory_basename = os.path.basename(output_directory)
        if not delete_origin or delete_origin and archive_basename != output_directory_basename:
            output_directory = os.path.join(output_directory, archive_basename)

        # Unpack
        status = self.file_to_file_unpack(input_file=input_file, output_directory=output_directory, file_type=file_type).status

        # Delete subarchives that have been unpacked
        if delete_origin:
            os.remove(input_file)

        # Recurse subdirectories on success and unpack all subarchives
        if status in successes.success_codes:
            if os.path.isdir(output_directory):
                for subarchive in ArchiveManager.get_archives(output_directory):
                    subarchive_parent_directory = os.path.dirname(subarchive)
                    self.recursively_unpack(input_file=subarchive, output_directory=subarchive_parent_directory, delete_origin=True)
            else:
                log.warning(f"Directory doesn't exist.\n{locals()}")
        else:
            log.warning(f"Bad status.\n{locals()}")





import glasswall
from glasswall.content_management.policies.policy import Policy


class Editor(Policy):
    """ A content management policy for Editor."""

    def __init__(self, default: str = "sanitise", config: dict = {}):
        self.default = default
        self.default_config_elements = [
            glasswall.content_management.config_elements.pdfConfig,
            glasswall.content_management.config_elements.pptConfig,
            glasswall.content_management.config_elements.sysConfig,
            glasswall.content_management.config_elements.tiffConfig,
            glasswall.content_management.config_elements.wordConfig,
            glasswall.content_management.config_elements.xlsConfig,
        ]
        self.config = config or {}

        super().__init__(
            default=self.default,
            default_config_elements=self.default_config_elements,
            config=self.config,
        )





import os
import setuptools


with open(os.path.join(os.path.dirname(__file__), "README.md"), "r") as f:
    long_description = f.read()


setuptools.setup(
    name="glasswall",
    description="Glasswall Python Wrapper",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="AngusWR",
    author_email="aroberts@glasswallsolutions.com",
    url="https://github.com/filetrust/glasswall-python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
    ],
    python_requires=">=3.6",
    include_package_data=True
)
